import { Component, ElementRef, HostListener, OnDestroy, OnInit, ViewEncapsulation ,NgModule,Input,Output,EventEmitter} from '@angular/core';
import { VgAPI } from '../../core/services/vg-api';
import { VgFullscreenAPI } from '../../core/services/vg-fullscreen-api';
import { Subscription } from 'rxjs';


@Component({
    selector: 'vg-fullscreen',
    encapsulation: ViewEncapsulation.None,
    template: `
        <div class="icon"
             [class.vg-icon-fullscreen]="!isFullscreen"
             [class.vg-icon-fullscreen_exit]="isFullscreen"
             tabindex="0"
             role="button"
             aria-label="fullscreen button"
             [attr.aria-valuetext]="ariaValue">
        </div>`,
    styles: [ `
        vg-fullscreen {
            -webkit-touch-callout: none;
            -webkit-user-select: none;
            -khtml-user-select: none;
            -moz-user-select: none;
            -ms-user-select: none;
            user-select: none;
            display: flex;
            justify-content: center;
            height: 50px;
            width: 50px;
            cursor: pointer;
            color: white;
            line-height: 50px;
        }

        vg-fullscreen .icon {
            pointer-events: none;
        }
    ` ]
})
export class VgFullscreen implements OnInit, OnDestroy {
    elem: HTMLElement;
    @Input() vgFor;
    target: Object;
    isFullscreen = false;
    @Input() type:any = 0;
    subscriptions: Subscription[] = [];

    ariaValue = 'normal mode';
    @Output() childbroadcast=new EventEmitter;
    constructor(ref: ElementRef, public API: VgAPI, public fsAPI: VgFullscreenAPI) {
        this.elem = ref.nativeElement;
        this.subscriptions.push(this.fsAPI.onChangeFullscreen.subscribe(this.onChangeFullscreen.bind(this)));
    }

    ngOnInit() {
        if (this.API.isPlayerReady) {
            this.onPlayerReady();
        }
        else {
            this.subscriptions.push(this.API.playerReadyEvent.subscribe(() => this.onPlayerReady()));
        }
    }

    onPlayerReady() {
        this.target = this.API.getMediaById(this.vgFor);
    }

    onChangeFullscreen(fsState: boolean) {
        this.ariaValue = fsState ? 'fullscren mode' : 'normal mode';
        this.isFullscreen = fsState;
        console.log(this.type );
        if(!this.isFullscreen){
            if(this.type == 0 && this.type != ''){
                console.log(this.isFullscreen);
                this.childbroadcast.emit(this.isFullscreen)
            }
        }
    }

    @HostListener('click')
    onClick() {
        this.changeFullscreenState();
        if(this.type == 0 && this.type != ''){
            console.log(this.isFullscreen);
            this.childbroadcast.emit(this.isFullscreen)
        }
    }

    @HostListener('keydown', ['$event'])
    onKeyDown(event: KeyboardEvent) {
        // On press Enter (13) or Space (32)
        if (event.keyCode === 13 || event.keyCode === 32) {
            event.preventDefault();
            this.changeFullscreenState();
        }
    }

    changeFullscreenState() {
        let element = this.target;

        if (this.target instanceof VgAPI) {
            element = null;
        }
        console.log(element);
        console.log('放大');

        this.fsAPI.toggleFullscreen(element);
    }

    ngOnDestroy() {
        this.subscriptions.forEach(s => s.unsubscribe());
    }
}
