import {
    Component, OnInit, Input, ElementRef, HostListener, ViewEncapsulation, OnDestroy,
    HostBinding
} from '@angular/core';
import { VgAPI } from '../core/services/vg-api';
import { VgStates } from '../core/states/vg-states';
import { Subscription } from 'rxjs';
import { VgFullscreenAPI } from '../core/services/vg-fullscreen-api';
import { VgControlsHidden } from '../core/services/vg-controls-hidden';

@Component({
    selector: 'vg-overlay-play',
    encapsulation: ViewEncapsulation.None,
    template: `<div class="vg-overlay-play"
                    [class.native-fullscreen]="isNativeFullscreen"
                    [class.controls-hidden]="areControlsHidden">
                   <div class="overlay-play-container" *ngIf="showPlayIcon == 0" 
                        [class.vg-icon-play_arrow]="getState() !== 'playing' && getState() !== 'ended'" [class.cuIcon-stop]="getState() === 'playing'">
                   </div>
                   <div class="overlay-play-container" *ngIf="showPlayIcon == 1" 
                        [class.vg-icon-play_arrow]="getState() !== 'playing'">
                   </div>
               </div>`,
    styles: [`
        vg-overlay-play {
            z-index: 200;
        }

        vg-overlay-play.is-buffering {
            display: none;
        }

        vg-overlay-play .vg-overlay-play {
            transition: all 0.5s;
            cursor: pointer;
            position: absolute;
            display: block;
            color: white;
            width: 100%;
            height: 100%;
            font-size: 80px;
            filter: alpha(opacity=60);
            opacity: 0.6;
        }

        vg-overlay-play .vg-overlay-play.native-fullscreen.controls-hidden {
            cursor: none;
        }

        vg-overlay-play .vg-overlay-play .overlay-play-container.vg-icon-play_arrow ,vg-overlay-play .vg-overlay-play .overlay-play-container.cuIcon-stop{
            pointer-events: none;
            width: 100%;
            height: 100%;
            position: absolute;
            display: flex;
            align-items: center;
            justify-content: center;
            font-size: 80px;
        }
        .controls-hidden .overlay-play-container{
            opacity: 0;
            -webkit-transition: opacity 1s;
            -khtml-transition: opacity 1s;
            -moz-transition: opacity 1s;
            -ms-transition: opacity 1s;
            transition: opacity 1s;
        }
        .overlay-play-container{
            opacity: 1;
            -webkit-transition: opacity 1s;
            -khtml-transition: opacity 1s;
            -moz-transition: opacity 1s;
            -ms-transition: opacity 1s;
            transition: opacity 1s;
        }
        vg-overlay-play .vg-overlay-play:hover {
            filter: alpha(opacity=100);
            opacity: 1;
        }
        vg-overlay-play .vg-overlay-play .isShow{
            display:none !important;
        }
        // vg-overlay-play .vg-overlay-play:hover .overlay-play-container.vg-icon-play_arrow:before {
        //     transform: scale(1.2);
        // }
    ` ]
})
export class VgOverlayPlay implements OnInit, OnDestroy {
    @Input() vgFor: string;
    @Input() vgAutohideTime = 3;
    @Input() showPlayIcon = 0;//0 为默认视频，播放完不显示播放按钮    1为评论视频，播放完显示按钮
    elem: HTMLElement;
    target: any;
    iconHide = false;
    iconShow = false;//点击显示
    isNativeFullscreen = false;
    areControlsHidden = false;
    private timer: any;
    private timers: any;
    subscriptions: Subscription[] = [];

    @HostBinding('class.is-buffering') isBuffering = false;

    constructor(ref: ElementRef, public API: VgAPI, public fsAPI: VgFullscreenAPI, private controlsHidden: VgControlsHidden) {
        this.elem = ref.nativeElement;
    }


    ngOnInit() {
        if (this.API.isPlayerReady) {
            this.onPlayerReady();
        }
        else {
            this.subscriptions.push(this.API.playerReadyEvent.subscribe(() => this.onPlayerReady()));
        }
    }

    onPlayerReady() {
        this.target = this.API.getMediaById(this.vgFor);
        this.subscriptions.push(this.fsAPI.onChangeFullscreen.subscribe(this.onChangeFullscreen.bind(this)));
        this.subscriptions.push(this.controlsHidden.isHidden.subscribe(this.onHideControls.bind(this)));
        this.subscriptions.push(
            this.target.subscriptions.bufferDetected.subscribe(
                isBuffering => this.onUpdateBuffer(isBuffering)
            )
        );
        // console.log(this.target);
    }

    onUpdateBuffer(isBuffering) {
        this.isBuffering = isBuffering;
    }

    onChangeFullscreen(fsState: boolean) {
        if (this.fsAPI.nativeFullscreen) {
            this.isNativeFullscreen = fsState;
        }
    }

    onHideControls(hidden: boolean) {
        this.areControlsHidden = hidden;
    }

    @HostListener('click')
    onClick() {
        console.log('点击显示');
        let state = this.getState();
        // console.log(this.iconHide);
        // console.log(this.iconShow);
        // console.log(state);
        // console.log(this.target);
        // console.log(this.target.state);
        // let vid = $('#' + this.vgFor)[0]
        // console.log('视频是否播放' + vid.paused);
        // if (!vid.paused && state == 'paused') {
        //     this.API.state = 'playing';
        //     state = 'playing';
        //     clearTimeout(this.timer);
        //     this.iconHide = false;
        //     this.target.pause();
        // }
        switch (state) {
            case VgStates.VG_PLAYING:
                // this.target.pause();
                //点击先显示暂停按钮，显示完之后，再点击才是暂停
                if (!this.iconHide && !this.iconShow) {
                    clearTimeout(this.timer);
                    this.iconHide = true;
                    this.hideAsync();
                } else {
                    clearTimeout(this.timer);
                    this.iconHide = false;
                    this.target.pause();
                    console.log('暂停');
                    // console.log('这里隐藏进度条');
                }
                break;

            case VgStates.VG_PAUSED:
                this.target.play();
                this.iconHide = false;
                this.iconShow = true;
                clearTimeout(this.timers);
                this.hideAsync();
                break;
            case VgStates.VG_ENDED:
                this.target.play();
                this.iconHide = false;
                this.iconShow = true;
                clearTimeout(this.timers);
                this.hideAsync();
                break;
        }
    }




    getState() {
        let state = VgStates.VG_PAUSED;

        if (this.target) {
            if (this.target.state instanceof Array) {
                for (let i = 0, l = this.target.state.length; i < l; i++) {
                    if (this.target.state[i] === VgStates.VG_PLAYING) {
                        state = VgStates.VG_PLAYING;
                        break;
                    }
                }
            }
            else {
                state = this.target.state;
            }
        }

        return state;
    }


    hide() {
        if (this.iconHide) {
            clearTimeout(this.timer);
            this.hideAsync();
        }
    }



    private hideAsync() {
        if (this.API.state === VgStates.VG_PLAYING) {
            this.timer = setTimeout(() => {
                this.iconHide = false;
            }, this.vgAutohideTime * 1000);
        } else {
            this.timers = setTimeout(() => {
                this.iconShow = false;
            }, this.vgAutohideTime * 1000);
        }
    }

    ngOnDestroy() {
        this.subscriptions.forEach(s => s.unsubscribe());
    }
}
